package ru.concerteza.util.string;

import java.util.HashMap;
import java.util.Map;

import static org.apache.commons.lang.StringUtils.isEmpty;

/**
 * Formatting utilities, borrowed from <a href="http://www.slf4j.org/">slf4j</a> project.
 *
 * @author alexey
 *         Date: 4/19/11
 * @see CtzFormatUtilsTest
 */
public class CtzFormatUtils {
    private static final String DELIM_STR = "{}";

    /**
     * {@code {}} placeholders will be replaced by positional arguments
     * (if they provided, left intact otherwise)
     *
     * @param messagePattern pattern with placeholders
     * @param args           positional arguments to replace placeholders
     * @return formatted message
     */
    public static String format(final String messagePattern, Object... args) {
        return formatArray(messagePattern, args);
    }

    // stripped from org.slf4j.helpers.MessageFormatter
    private static String formatArray(final String messagePattern, final Object[] argArray) {
        if(isEmpty(messagePattern)) return "null";
        if(null == argArray) return messagePattern;

        int i = 0;
        int j;
        StringBuilder sb = new StringBuilder(messagePattern.length() + 50);
        for(Object anArgArray : argArray) {
            j = messagePattern.indexOf(DELIM_STR, i);
            if(j == -1) {
                // no more variables
                if(i == 0) { // this is a simple string
                    return messagePattern;
                } else { // add the tail string which contains no variables and return
                    // the result.
                    sb.append(messagePattern.substring(i, messagePattern.length()));
                    return sb.toString();
                }
            } else {
                sb.append(messagePattern.substring(i, j));
                deeplyAppendParameter(sb, anArgArray, new HashMap<Object[], Object>());
                i = j + 2;
            }
        }
        // append the characters following the last {} pair.
        sb.append(messagePattern.substring(i, messagePattern.length()));
        return sb.toString();
    }

    // special treatment of array values was suggested by 'lizongbo'
    private static void deeplyAppendParameter(StringBuilder sbuf, Object o, Map<Object[], Object> seenMap) {
        if(o == null) {
            sbuf.append("null");
            return;
        }
        if(!o.getClass().isArray()) {
            sbuf.append(o.toString());
        } else {
            // check for primitive array types because they
            // unfortunately cannot be cast to Object[]
            if(o instanceof boolean[]) {
                booleanArrayAppend(sbuf, (boolean[]) o);
            } else if(o instanceof byte[]) {
                byteArrayAppend(sbuf, (byte[]) o);
            } else if(o instanceof char[]) {
                charArrayAppend(sbuf, (char[]) o);
            } else if(o instanceof short[]) {
                shortArrayAppend(sbuf, (short[]) o);
            } else if(o instanceof int[]) {
                intArrayAppend(sbuf, (int[]) o);
            } else if(o instanceof long[]) {
                longArrayAppend(sbuf, (long[]) o);
            } else if(o instanceof float[]) {
                floatArrayAppend(sbuf, (float[]) o);
            } else if(o instanceof double[]) {
                doubleArrayAppend(sbuf, (double[]) o);
            } else {
                objectArrayAppend(sbuf, (Object[]) o, seenMap);
            }
        }
    }

    private static void objectArrayAppend(StringBuilder sbuf, Object[] a, Map<Object[], Object> seenMap) {
        sbuf.append('[');
        if(!seenMap.containsKey(a)) {
            seenMap.put(a, null);
            final int len = a.length;
            for(int i = 0; i < len; i++) {
                deeplyAppendParameter(sbuf, a[i], seenMap);
                if(i != len - 1)
                    sbuf.append(", ");
            }
            // allow repeats in siblings
            seenMap.remove(a);
        } else {
            sbuf.append("...");
        }
        sbuf.append(']');
    }

    private static void booleanArrayAppend(StringBuilder sbuf, boolean[] a) {
        sbuf.append('[');
        final int len = a.length;
        for(int i = 0; i < len; i++) {
            sbuf.append(a[i]);
            if(i != len - 1)
                sbuf.append(", ");
        }
        sbuf.append(']');
    }

    private static void byteArrayAppend(StringBuilder sbuf, byte[] a) {
        sbuf.append('[');
        final int len = a.length;
        for(int i = 0; i < len; i++) {
            sbuf.append(a[i]);
            if(i != len - 1)
                sbuf.append(", ");
        }
        sbuf.append(']');
    }

    private static void charArrayAppend(StringBuilder sbuf, char[] a) {
        sbuf.append('[');
        final int len = a.length;
        for(int i = 0; i < len; i++) {
            sbuf.append(a[i]);
            if(i != len - 1)
                sbuf.append(", ");
        }
        sbuf.append(']');
    }

    private static void shortArrayAppend(StringBuilder sbuf, short[] a) {
        sbuf.append('[');
        final int len = a.length;
        for(int i = 0; i < len; i++) {
            sbuf.append(a[i]);
            if(i != len - 1)
                sbuf.append(", ");
        }
        sbuf.append(']');
    }

    private static void intArrayAppend(StringBuilder sbuf, int[] a) {
        sbuf.append('[');
        final int len = a.length;
        for(int i = 0; i < len; i++) {
            sbuf.append(a[i]);
            if(i != len - 1)
                sbuf.append(", ");
        }
        sbuf.append(']');
    }

    private static void longArrayAppend(StringBuilder sbuf, long[] a) {
        sbuf.append('[');
        final int len = a.length;
        for(int i = 0; i < len; i++) {
            sbuf.append(a[i]);
            if(i != len - 1)
                sbuf.append(", ");
        }
        sbuf.append(']');
    }

    private static void floatArrayAppend(StringBuilder sbuf, float[] a) {
        sbuf.append('[');
        final int len = a.length;
        for(int i = 0; i < len; i++) {
            sbuf.append(a[i]);
            if(i != len - 1)
                sbuf.append(", ");
        }
        sbuf.append(']');
    }

    private static void doubleArrayAppend(StringBuilder sbuf, double[] a) {
        sbuf.append('[');
        final int len = a.length;
        for(int i = 0; i < len; i++) {
            sbuf.append(a[i]);
            if(i != len - 1)
                sbuf.append(", ");
        }
        sbuf.append(']');
    }
}
